/*
    CharityWorld: AllPortionDemandAgents.m
    Copyright (C) 2003  Macaulay Institute

    This file is part of CharityWorld, a simple agent-based model to
    demonstrate the effects of errors in floating point arithmetic on
    emergent outcomes.

    CharityWorld is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    CharityWorld is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/* 

This file contains the implementation for two classes:
PortionDemandAgent and ExtAvgPortionDemandAgent.
They only differ on the class they inherit from.
The implementation of the methods that they override 
is exactly the same for both classes.
(A dodgy way of doing multiple inheritance )

*/

#import "AllPortionDemandAgents.h"
#import "DoubleSimple.h"
#import "Verbosity.h"
#import "Bank.h"

/*

PortionDemandAgent.m

*/

@implementation PortionDemandAgent

-ask { 
  id <Index> ix = nil;
  Agent *nbr;

  if([Verbosity showAgentAsking]) {
    printf("Agent (%d, %d) is asking from their neighbours:\n", x + 1, y + 1);
  }
  for(ix = [nbrs begin: scratchZone], [ix setLoc: End],
	nbr = (Agent *)[ix prev];
      [ix getLoc] == Member;
      nbr = (Agent *)[ix prev]) {
    if([Verbosity showAgentAsking]) {
      printf("\tAgent (%d, %d) has wealth ", x + 1, y + 1);
      [wealth writeAccurately: stdout];
      printf("\n");
      printf("\tConsidering neighbour (%d, %d) with wealth ", [nbr getX] + 1,
	     [nbr getY] + 1);
      [[[nbr getWealth] writeAccurately: stdout] free];
      printf("...\n");
    }
    if([self beggarCondition]) {
      if([self askingCondition: nbr]) {
	DoubleSimple *begAmount = [localWealth copy];
	[begAmount idiv: 1 + numberOfNbrs];
	[begAmount sub: wealth];
	[begAmount mul: beg];
	if([Verbosity showAgentAsking]) {
	  printf("\t...meets asking condition so telling them to respond "
		 "with ");
	  [begAmount writeAccurately: stdout];
	  printf("\n");
	}
        [nbr respondTo: self askingFor: begAmount];
	[begAmount free];
      }
      else {
	if([Verbosity showAgentAsking]) {
	  printf("\t...does not meet asking condition so ignoring them\n");
	}
      }
    }
    else {
      if([Verbosity showAgentAsking]) {
	printf("\t...but I no longer satisfy the beggar condition!\n");
      }
    }
  }
  [ix drop];
  for(ix = [nbrs begin: scratchZone], [ix setLoc: End],
	nbr = (Agent *)[ix prev];
      [ix getLoc] == Member;
      nbr = (Agent *)[ix prev]) {
    if(![nbr isWaiting]) {
      [nbr rescheduleCheck];
    }
  }
  [ix drop];
  return self;
}

@end


/*

ExtAvgPortionDemandAgent.m

*/

@implementation ExtAvgPortionDemandAgent

-ask { 
  id <Index> ix = nil;
  Agent *nbr;

  if([Verbosity showAgentAsking]) {
    printf("Agent (%d, %d) is asking from their neighbours:\n", x + 1, y + 1);
  }
  for(ix = [nbrs begin: scratchZone], [ix setLoc: End],
	nbr = (Agent *)[ix prev];
      [ix getLoc] == Member;
      nbr = (Agent *)[ix prev]) {
    if([Verbosity showAgentAsking]) {
      printf("\tAgent (%d, %d) has wealth ", x + 1, y + 1);
      [wealth writeAccurately: stdout];
      printf("\n");
      printf("\tConsidering neighbour (%d, %d) with wealth ", [nbr getX] + 1,
	     [nbr getY] + 1);
      [[[nbr getWealth] writeAccurately: stdout] free];
      printf("...\n");
    }
    if([self beggarCondition]) {
      if([self askingCondition: nbr]) {
	DoubleSimple *begAmount = [localWealth copy];
	[begAmount idiv: numberOfNbrs];
	[begAmount sub: wealth];
	[begAmount mul: beg];
	if([Verbosity showAgentAsking]) {
	  printf("\t...meets asking condition so telling them to respond "
		 "with ");
	  [begAmount writeAccurately: stdout];
	  printf("\n");
	}
        [nbr respondTo: self askingFor: begAmount];
	[begAmount free];
      }
      else {
	if([Verbosity showAgentAsking]) {
	  printf("\t...does not meet asking condition so ignoring them\n");
	}
      }
    }
    else {
      if([Verbosity showAgentAsking]) {
	printf("\t...but I no longer satisfy the beggar condition!\n");
      }
    }
  }
  [ix drop];
  for(ix = [nbrs begin: scratchZone], [ix setLoc: End],
	nbr = (Agent *)[ix prev];
      [ix getLoc] == Member;
      nbr = (Agent *)[ix prev]) {
    if(![nbr isWaiting]) {
      [nbr rescheduleCheck];
    }
  }
  [ix drop];
  return self;
}

@end

/*

GreedyPortionDemandAgent.m

*/

@implementation GreedyPortionDemandAgent

-ask { 
  id <Index> ix = nil;
  Agent *nbr;

  if([Verbosity showAgentAsking]) {
    printf("Agent (%d, %d) is asking from their neighbours:\n", x + 1, y + 1);
  }
  for(ix = [nbrs begin: scratchZone], [ix setLoc: End],
	nbr = (Agent *)[ix prev];
      [ix getLoc] == Member;
      nbr = (Agent *)[ix prev]) {
    if([Verbosity showAgentAsking]) {
      printf("\tAgent (%d, %d) has wealth ", x + 1, y + 1);
      [wealth writeAccurately: stdout];
      printf("\n");
      printf("\tConsidering neighbour (%d, %d) with wealth ", [nbr getX] + 1,
	     [nbr getY] + 1);
      [[[nbr getWealth] writeAccurately: stdout] free];
      printf("...\n");
    }
    if([self beggarCondition]) {
      if([self askingCondition: nbr]) {
	DoubleSimple *begAmount = [nbr getWealth];
	[begAmount add: wealth];
	[begAmount idiv: 2];
	[begAmount sub: wealth];
	[begAmount mul: beg];
	if([Verbosity showAgentAsking]) {
	  printf("\t...meets asking condition so telling them to respond "
		 "with ");
	  [begAmount writeAccurately: stdout];
	  printf("\n");
	}
        [nbr respondTo: self askingFor: begAmount];
	[begAmount free];
      }
      else {
	if([Verbosity showAgentAsking]) {
	  printf("\t...does not meet asking condition so ignoring them\n");
	}
      }
    }
    else {
      if([Verbosity showAgentAsking]) {
	printf("\t...but I no longer satisfy the beggar condition!\n");
      }
    }
  }
  [ix drop];
  for(ix = [nbrs begin: scratchZone], [ix setLoc: End],
	nbr = (Agent *)[ix prev];
      [ix getLoc] == Member;
      nbr = (Agent *)[ix prev]) {
    if(![nbr isWaiting]) {
      [nbr rescheduleCheck];
    }
  }
  [ix drop];
  return self;
}

@end
