/*
    CharityWorld: AllSimpleSuppluAgents.m
    Copyright (C) 2003  Macaulay Institute

    This file is part of CharityWorld, a simple agent-based model to
    demonstrate the effects of errors in floating point arithmetic on
    emergent outcomes.

    CharityWorld is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    CharityWorld is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/* 

This file contains the implementation for four classes:
SimpleSupplyAgent, ExtAvgSimpleSupplyAgent, 
MeanSimpleSupplyAgent, and ExtMeanSimpleSupplyAgent.
They only differ on the class they inherit from.
The implementation of the methods that they override 
is exactly the same for all the classes.
(A dodgy way of doing multiple inheritance )

*/


#import "AllSimpleSupplyAgents.h"
#import "ModelSwarm.h"
#import "MergeSorter.h"
#import "Verbosity.h"
#import "Shuffler.h"
#import "DoubleSimple.h"
#import <random.h>



/*

CharityWorld

SimpleSupplyAgent.m

*/

@implementation SimpleSupplyAgent

-step {
  if([Verbosity showAgentStepping]) printf("Stepping agent (%d, %d)\n", x + 1,
					   y + 1);
  // sort the neighbours in increasing order of wealth
  [Shuffler shuffleList: nbrs];
  [MergeSorter mergeSortDouble: nbrs withCloningSelector: M(getWealth)];
  
  if([self donorCondition]) {
    if([Verbosity showAgentSteppingDetail]) {
      printf("Agent (%d, %d) is offering because they meet donor condition\n",
	     x + 1, y + 1);
    }
    [self offer];
  }
  else {
    if([Verbosity showAgentSteppingDetail]) {
      printf("Agent (%d, %d) is not offering because they don't meet "
	     "donor condition\n", x + 1, y + 1);
    }
  }

  [self rescheduleCheck];
  return self;
}

-ask { 
  printf("WARNING: SimpleSupplyAgent does not *do* asking!\n");
  return self;
}

-(void)respondTo: (Agent *)a askingFor: (DoubleSimple *)amount {
  printf("WARNING: SimpleSupplyAgent does not respond to requests for "
	 "money!\n");
}

-(void)reschedule {
  waiting = YES;
  if([self donorCondition]) {
    if([Verbosity showAgentRescheduling]) {
      printf("(%d, %d) queued to donor list\n", x + 1, y + 1);
    }
    [model queueDonor: self];
  }
  else {
    if([Verbosity showAgentRescheduling]) {
      printf("(%d, %d) not queued to any list\n", x + 1, y + 1);
    }
    waiting = NO;
  }
}  

@end


/*

CharityWorld

ExtAvgSimpleSupplyAgent.m

*/

@implementation ExtAvgSimpleSupplyAgent

-step {
  if([Verbosity showAgentStepping]) printf("Stepping agent (%d, %d)\n", x + 1,
					   y + 1);
  // sort the neighbours in increasing order of wealth
  [Shuffler shuffleList: nbrs];
  [MergeSorter mergeSortDouble: nbrs withCloningSelector: M(getWealth)];
  
  if([self donorCondition]) {
    if([Verbosity showAgentSteppingDetail]) {
      printf("Agent (%d, %d) is offering because they meet donor condition\n",
	     x + 1, y + 1);
    }
    [self offer];
  }
  else {
    if([Verbosity showAgentSteppingDetail]) {
      printf("Agent (%d, %d) is not offering because they don't meet "
	     "donor condition\n", x + 1, y + 1);
    }
  }

  [self rescheduleCheck];
  return self;
}


-ask { 
  printf("WARNING: ExtAvgSimpleSupplyAgent does not *do* asking!\n");
  return self;
}

-(void)respondTo: (Agent *)a askingFor: (DoubleSimple *)amount {
  printf("WARNING: ExtAvgSimpleSupplyAgent does not respond to requests for "
	 "money!\n");
}

-(void)reschedule {
  waiting = YES;
  if([self donorCondition]) {
    if([Verbosity showAgentRescheduling]) {
      printf("(%d, %d) queued to donor list\n", x + 1, y + 1);
    }
    [model queueDonor: self];
  }
  else {
    if([Verbosity showAgentRescheduling]) {
      printf("(%d, %d) not queued to any list\n", x + 1, y + 1);
    }
    waiting = NO;
  }
}  

@end

/*

CharityWorld

MeanSimpleSupplyAgent.m

*/

@implementation MeanSimpleSupplyAgent

-step {
  if([Verbosity showAgentStepping]) printf("Stepping agent (%d, %d)\n", x + 1,
					   y + 1);
  // sort the neighbours in increasing order of wealth
  [Shuffler shuffleList: nbrs];
  [MergeSorter mergeSortDouble: nbrs withCloningSelector: M(getWealth)];
  
  if([self donorCondition]) {
    if([Verbosity showAgentSteppingDetail]) {
      printf("Agent (%d, %d) is offering because they meet donor condition\n",
	     x + 1, y + 1);
    }
    [self offer];
  }
  else {
    if([Verbosity showAgentSteppingDetail]) {
      printf("Agent (%d, %d) is not offering because they don't meet "
	     "donor condition\n", x + 1, y + 1);
    }
  }

  [self rescheduleCheck];
  return self;
}


-ask { 
  printf("WARNING: MeanSimpleSupplyAgent does not *do* asking!\n");
  return self;
}

-(void)respondTo: (Agent *)a askingFor: (DoubleSimple *)amount {
  printf("WARNING: MeanSimpleSupplyAgent does not respond to requests for "
	 "money!\n");
}

-(void)reschedule {
  waiting = YES;
  if([self donorCondition]) {
    if([Verbosity showAgentRescheduling]) {
      printf("(%d, %d) queued to donor list\n", x + 1, y + 1);
    }
    [model queueDonor: self];
  }
  else {
    if([Verbosity showAgentRescheduling]) {
      printf("(%d, %d) not queued to any list\n", x + 1, y + 1);
    }
    waiting = NO;
  }
}  

@end

/*

CharityWorld

ExtMeanSimpleSupplyAgent.m

*/


@implementation ExtMeanSimpleSupplyAgent

-step {
  if([Verbosity showAgentStepping]) printf("Stepping agent (%d, %d)\n", x + 1,
					   y + 1);
  // sort the neighbours in increasing order of wealth
  [Shuffler shuffleList: nbrs];
  [MergeSorter mergeSortDouble: nbrs withCloningSelector: M(getWealth)];
  
  if([self donorCondition]) {
    if([Verbosity showAgentSteppingDetail]) {
      printf("Agent (%d, %d) is offering because they meet donor condition\n",
	     x + 1, y + 1);
    }
    [self offer];
  }
  else {
    if([Verbosity showAgentSteppingDetail]) {
      printf("Agent (%d, %d) is not offering because they don't meet "
	     "donor condition\n", x + 1, y + 1);
    }
  }

  [self rescheduleCheck];
  return self;
}


-ask { 
  printf("WARNING: ExtMeanSimpleSupplyAgent does not *do* asking!\n");
  return self;
}

-(void)respondTo: (Agent *)a askingFor: (DoubleSimple *)amount {
  printf("WARNING: ExtMeanSimpleSupplyAgent does not respond to requests for "
	 "money!\n");
}

-(void)reschedule {
  waiting = YES;
  if([self donorCondition]) {
    if([Verbosity showAgentRescheduling]) {
      printf("(%d, %d) queued to donor list\n", x + 1, y + 1);
    }
    [model queueDonor: self];
  }
  else {
    if([Verbosity showAgentRescheduling]) {
      printf("(%d, %d) not queued to any list\n", x + 1, y + 1);
    }
    waiting = NO;
  }
}  

@end

