/*
    CharityWorld: PaperModelSwarm.m
    Copyright (C) 2003  Macaulay Institute

    This file is part of CharityWorld, a simple agent-based model to
    demonstrate the effects of errors in floating point arithmetic on
    emergent outcomes.

    CharityWorld is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    CharityWorld is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/

#import "PaperModelSwarm.h"
#import "Verbosity.h"
#import "DoubleSimple.h"
#import <string.h>
#import <stdlib.h>

@implementation PaperModelSwarm

-buildObjects {
  model = [ModelSwarm create: self];
  [model setGenerosityMinStr: generosityStr maxStr: generosityStr];
  [model setBegMinStr: "0.0" maxStr: "0.0"];
  [model setJackpotStr: jackpotStr];
  [model setInitialWealthStr: initialWealthStr];
  [model setAgentClassStr: agentClassStr];
  [model setWinnerCoord: winnerCoord];
  if(strcmp(loserChooser, "all") == 0 || strcmp(loserChooser, "none") == 0) {
    [model setLoserChooser: loserChooser];
  }
  else {
    fprintf(stderr, "Invalid setting for loserChooser: %s\n", loserChooser);
    abort();
  }
  [model setQueueClassStr: "QueueSIRO"];
  [model setEnvShape: "planar" neighbourhood: "moore" xSize: xSize ySize: ySize
	 radius: 1];
  [model buildObjects];
  return self;
}

-buildActions {
  return [model buildActions];
}

-activateIn: swarmContext {
  return [model activateIn: swarmContext];
}

-(void)drop {
  [model drop];
  [super drop];
}

-(retval_t)forward: (SEL)sel : (arglist_t)args {
  retval_t retval;
  if([Verbosity showPaperModelMessageForwarding]) {
    printf("PaperModel forwarding message %s to Model\n", sel_get_name(sel));
  }
  retval = [model performv: sel : args];
  if([Verbosity showPaperModelMessageForwarding]) {
    printf("PaperModel received %p from [model %s]\n", retval,
	   sel_get_name(sel));
  }
  return retval;
}

-(int)getXSize {
  return [model getXSize];
}

-(int)getYSize {
  return [model getYSize];
}


-(double)getJackpotForDisplayOnly {
  return [model getJackpotForDisplayOnly];
}

-(double)getInitialWealthForDisplayOnly {
  return [model getInitialWealthForDisplayOnly];
}

-(double)getMinWealthForDisplayOnly {
  return [model getMinWealthForDisplayOnly];
}

-(double)getMaxWealthForDisplayOnly {
  return [model getMaxWealthForDisplayOnly];
}


-(void)queueBeggar: (Agent *)a {
  [model queueBeggar: a];
}

-(void)queueDonor: (Agent *)a {
  [model queueDonor: a];
}

-getWorld {
  return [model getWorld];
}

-getAgents {
  return [model getAgents];
}

-getBeggars {
  return [model getBeggars];
}

-getDonors {
  return [model getDonors];
}

-(DoubleSimple *)getJackpot {
  return [model getJackpot];
}

-(DoubleSimple *)getInitialWealth {
  return [model getInitialWealth];
}

@end
